/*  nauta for Palm: displays and logs serial data from GPS sensors
    Copyright (C) 2002 D.T.Ineiev, super V 93
    Email: <ineiev@yahoo.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include"navdata.h"
#include"InitNavData.h"
#include"setnavdata.h"
#include"nautaStr.h"
#include"events.h"
#include"FieldFunc.h"
#include"prefs.h"
#include"proplist.h"
#include"sensors.h"
#include"screen.h"
#include<PalmOS.h>
#define CustomValLen 19
struct Fixed{char v;Int32 x;};static struct Fixed Alt,Speed,Course,HDOP,Heading;
char LatReceived=0,LonReceived=0,UTCReceived=0,SatsReceived=0;
static void SetFixed(struct Fixed*a,Int32 x){a->v=!0;a->x=x;}
static void InitFixed(struct Fixed*a){a->v=0;}
static struct Angle Lat,Lon;
#define scrn   5
#define scrl   289
static char scrol[scrn][scrl];static int scrolll[scrn],caput,cauda;
static void inc(int*x){*x=*x<scrn-1?*x+1:0;}
void SetScrollData(char*s,int n)
{MemMove(scrol[caput],s,scrolll[caput]=n<scrl?n:scrl);
 inc(&caput);if(cauda==caput)inc(&cauda);
}
int GetScrollData(char**s)
{int r=scrolll[cauda];if(cauda==caput)return 0;*s=scrol[cauda];inc(&cauda);
 return r;
}
static char CustomVals[CustomValLen*9];
static char satsInView=0,quality=0;
int CurrColor(void){return quality*3+(satsInView>3?2:(satsInView>2?1:0));}
static char FixQuality[17];
char*GetFixQuality()
{switch(CurrColor())
 {case 0:GetStrResource(FixQuality,kidStrNoFix);break;
  case 1:GetStrResource(FixQuality,kidStrWhat);break;
  case 2:GetStrResource(FixQuality,kidStrWhat);break;
  case 3:return"0D GPS";case 4:return"2D GPS";
  case 5:return"2D/3D GPS";case 6:return"0D DGPS";
  case 7:return"2D DGPS";case 8:return"2D/3D DGPS";
  default:GetStrResource(FixQuality,kidStrGood);break;
 }return FixQuality;
}
int GetSatsInView(void){if(SatsReceived)return satsInView;else return-1;}
void SetWarning(char*s)
{quality=*s=='V'?0:(quality?quality:3);
 if(!quality)satsInView=satsInView>2?0:satsInView;SetFieldsColor(CurrColor());
}
void SetQuality(char*s){quality=*s-'0';SetFieldsColor(CurrColor());}
void SetSatsInView(char*s)
{if(s[1]>='0'&&s[1]<='9')satsInView=(*s-'0')*10+s[1]-'0';else satsInView=*s-'0';
 SetFieldsColor(CurrColor());SatsReceived=!0;
}
char*GetCustomVal(int i){return CustomVals+(i<9?i*CustomValLen:0);}
void SetCustomVal(int n,char*s)
{char*d=GetCustomVal(n);int i;
 for(i=0;i++<CustomValLen&&*s&&*s!=','&&*s!='*';*d++=*s++);*d=0;
}
Err NavInit(void)
{int i;Lat.deg=Lat.min=Lat.sec=Lat.frac=Lon.deg=Lon.min=Lon.sec=Lon.frac=0;
 InitFixed(&Alt);InitFixed(&Speed);InitFixed(&Course);InitFixed(&HDOP);
 InitFixed(&Heading);
 cauda=caput=0;for(i=0;i<CustomValLen*9;CustomVals[i++]=0);return 0;
}
static UInt16 Sensor=0;UInt32 PropSensor=0,utc=0;
static Int32 difft=-170000l;//Palm-UTC
Int32 GetPalmMinusUTC(void){return difft;}
void SetSensor(char*s){Sensor=(((UInt16)*s)<<8)+s[1];}
static void GetPosAngle(char*s,const struct Angle*a,const char*ns)
{int deg=a->deg<0?-a->deg:a->deg;
 *s++=(deg>=100)?'1':' ';*s++=(deg/10)%10+'0';*s++=deg%10+'0';*s++='\xb0'; 
 *s++=a->min/10+'0';*s++=a->min%10+'0';*s++='\'';
 *s++=a->sec/10+'0';*s++=a->sec%10+'0';*s++='.';
 *s++=a->frac/100+'0';*s++=(a->frac-(a->frac/100)*100)/10+'0';
 *s++=a->frac%10+'0';*s++='\"';if(a->deg<0)*s++=ns[1];else*s++=*ns;*s=0;
}
void GetLat(char*s){if(LatReceived)GetPosAngle(s,&Lat,"NS");else *s=0;}
void GetLon(char*s){if(LonReceived)GetPosAngle(s,&Lon,"EW");else *s=0;}
void SetLat(struct Angle*a){Lat=*a;LatReceived=!0;}
void SetLon(struct Angle*a){Lon=*a;LonReceived=!0;}
static void GetFixed(char*s,struct Fixed*A)
{Int32 x,x10;if(!(A->v)){*s=0;return;}x=A->x/FixedScale;
 s+=StrLen(StrIToA(s,x));*s++='.';x=A->x%FixedScale;
 for(x10=FixedScale;x10>10;*s++=x/x10+'0'){x%=x10;x10/=10;}*s++=x%10+'0';*s=0;
}
void GetAlt(char*s){GetFixed(s,&Alt);}void SetAlt(Int32 x){SetFixed(&Alt,x);}
void GetSpeed(char*s){GetFixed(s,&Speed);}
void SetSpeed(Int32 x){SetFixed(&Speed,x);}
void GetCourse(char*s){GetFixed(s,&Course);}
void SetCourse(Int32 x){SetFixed(&Course,x);}
void GetHeading(char*s){GetFixed(s,&Heading);}
void SetHeading(Int32 x){SetFixed(&Heading,x);}
void GetHDOP(char*s){GetFixed(s,&HDOP);}
void SetHDOP(Int32 x){SetFixed(&HDOP,x);}
void GetUTC(char*s)
{Int32 x=utc/FixedScale,x10;if(!UTCReceived){*s=0;return;}
 x/=3600;*s++=x/10+'0';*s++=x%10+'0';*s++=':';x=(utc/FixedScale)%3600;
 *s++=x/600+'0';*s++=(x/60)%10+'0';*s++=':';x%=60;
 *s++=x/10+'0';*s++=x%10+'0';*s++='.';x=utc%FixedScale;
 for(x10=FixedScale;x10>10;*s++=x/x10+'0'){x%=x10;x10/=10;}*s++=x%10+'0';*s=0;
}void SetUTC(UInt32 x){utc=x;UTCReceived=!0;}
char*GetSensor(void)    
{if((Sensor>>8)&0xFF=='P')return ProprietarySensor(PropSensor);
 return GetGeneralSensor(Sensor);
}              
static void adjusttime(Int32 dt)
{DateTimeType d;UInt32 t;if(!syncEnabled())return;
 t=TimGetSeconds();TimSecondsToDateTime(t,&d);TimAdjust(&d,dt);
 t=TimDateTimeToSeconds(&d);TimSetSeconds(t);
}
void setdifft(UInt32 tic)
{const Int32 SecsPDay=24*3600l;
 difft=(tic-utc/FixedScale+SecsPDay/2)%SecsPDay-SecsPDay/2;adjusttime(-difft);
}
